﻿using System;
using System.IO;
using System.Net;
using System.Threading.Tasks;
using Clifton.Core.Pipes;
using Ninject.Activation.Strategies;
using TourDataManager;
using TourDataManager.Entities;

namespace TourDataManagerConsoleApplication {
    internal class Program {
        
        const string PersistentPath = "C:\\Users\\Bigantounity\\AppData\\LocalLow\\Biganto\\StandalonePlayer";
        
        const string defaultLogin = "demo@biganto.ru";
        const string defaultPassword = "demo";

        public static TourDataManager.TourDataManagerPlugin TourDataManagerPlugin;
        
        public static void Main(string[] args){

            //TourDataManagerPlugin = new TourDataManager.TourDataManagerPlugin(PersistentPath);
            CreateServer("Test3");
            //MainWindow();


            //tourDataManager.Login(defaultLogin, defaultPassword);
            //AsyncMethod(tourDataManager);
            
            Console.Read();
        }
        
        private static ServerPipe CreateServer(string serverName)
        {
            ServerPipe serverPipe = new ServerPipe(serverName, p => p.StartStringReaderAsync());

            serverPipe.DataReceived += (sndr, args) => {
                Console.WriteLine($"Server pipe /DataRecieved : {args.String}");
            };


            serverPipe.Connected += (sndr, args) => {
                Console.WriteLine($"Server pipe /Connected : {args}");
            };

            serverPipe.PipeClosed += (sndr, args) => {
                Console.WriteLine($"Server pipe /Closed : {args}");
            };

            return serverPipe;
        }

        public static void MainWindow(){
            Debug.Log("Commands : \n0 - Auth\n1 - Show estates list\n2 - exit");
            var ch = Debug.ReadKey();

            switch (ch){
                case '0' :
                    Debug.Log("Login : ");
                    var login = Console.ReadLine();
                    Debug.Log("Password : ");
                    var pass = Console.ReadLine();
                    
                    var result = TourDataManagerPlugin.Login(login,pass);
                    Debug.Log($"Auth result : {result.Item1} {result.Item2}");
                    MainWindow();
                    break;
                case '1' :
                    var estates = TourDataManagerPlugin.FetchEstatesAsync().Result;
                    foreach (var estate in estates){
                        Debug.Log(estate);
                    }
                    EstateWindow();
                    break;
                case '2' : Environment.Exit(0); break;
            }
        }

        public static void EstateWindow(){
            Debug.Log("Commands : \n0 - Fetch tours of estate\n1 - back\n2 - exit");
            var ch = Debug.ReadKey();

            switch (ch){
                case '0' :
                    Debug.Log("Estate id : ");
                    var id = Console.ReadLine();
                    var tours = TourDataManagerPlugin.FetchTourPreviewsAsync(Int32.Parse(id)).Result;
                    foreach (var tour in tours){
                        Debug.Log(tour);
                    }
                    EstateWindow();
                    break;
                case '1' : MainWindow(); break;
                case '2' : Environment.Exit(0); break;
            }
        }


        public static async void AsyncMethod(TourDataManager.TourDataManagerPlugin plugin){
            try{
                var estates = await plugin.FetchEstatesAsync();
                Tour[] tours = null;
                foreach (var estate in estates){
                    Debug.Log(estate);
                    tours = await plugin.FetchTourPreviewsAsync(estate.Id);
                    foreach (var tour in tours){
                        Debug.Log(tour);
                    }
                }

                System.Diagnostics.Debug.Assert(tours != null, nameof(tours) + " != null");
                var contentLoadingTask = plugin.DownloadTourContent(tours[2].Id);
                
                var x = Task.Factory
                    .StartNew(() => { contentLoadingTask.Run(); })
                    .ContinueWith(task => {
                        Debug.Log("Done");
                    });

            } catch (Exception e){
                Debug.Log(e);
            }


        }

    }
}